# encoding: utf-8

module Mysqlknife
  class Configs
    include Singleton

    attr_writer :connection
    attr_reader :mysql,
                :ssh,
                :connections,
                :color,
                :name,
                :host,
                :commands

    def initialize
      # Define path for config file:
      if ENV['ENV'] == 'test'
        path = File.expand_path(File.join(Dir.pwd, '.db.yml'))
      else
        path = File.expand_path("#{Dir.home}/.db.yml")
      end

      # Load config file:
      @configs = YAML.load_file(path) if File.exist?(path)

      if @configs.nil?
        puts "Not exist config file in #{path}"
        exit 1
      end

      # Define generic variables:
      @ssh   = {}
      @mysql = {}

      @ssh[:host]     = @configs['ssh']['host']
      @ssh[:port]     = @configs['ssh']['port']
      @ssh[:user]     = @configs['ssh']['user']
      @ssh[:password] = @configs['ssh']['password']
      @ssh[:key]      = @configs['ssh']['keys']
      @connections    = @configs['connections'].keys.sort
      @commands       = @configs['commands'].keys.sort
    end

    def method_missing(name, *args)
      method = name.to_s.split(/_/)
      if method.first == 'db'
        param = method[1]
        conn  = args[0]

        if @configs['connections'].include?(conn)
          @configs['connections'][conn][param]
        end
      end
    end

    def connection(name)
      @name             = name
      @color            = db_color(@name)
      @ssh[:use]        = db_ssh(@name)
      @mysql[:host]     = db_host(@name)
      @mysql[:port]     = db_port(@name)
      @mysql[:username] = db_username(@name)
      @mysql[:password] = db_password(@name)
      @mysql[:slaves]   = db_slaves(@name)
    end

    def hosts
      (@mysql[:slaves] = []) if @mysql[:slaves].nil?
      (@mysql[:host]   = '') if @mysql[:host].nil?

      (@mysql[:host] + ',' + @mysql[:slaves].join(',')).split(',')
    end

    def select(name)
      if hosts.is_a?(Array)
        hosts.each do |host|
          return @host = host if host.include?(name)
        end
      end
      nil
    end

    def command(name)
      if @configs['commands'].key?(name)
        @configs['commands'][name]
      end
    end
  end
end
